package ca.recrown.islandsurvivalcraft;

import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.event.HandlerList;
import org.bukkit.event.Listener;
import org.bukkit.generator.ChunkGenerator;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;

import ca.recrown.islandsurvivalcraft.interaction.commands.CommandProcessor;
import ca.recrown.islandsurvivalcraft.world.WorldInfoManager;
import ca.recrown.islandsurvivalcraft.world.generation.GeneratorModes;

public class IslandSurvivalCraft extends JavaPlugin implements Listener {
    private PluginManager pluginManager;
    private WorldInfoManager worldInfoManager;
    private CommandProcessor commandProcessor;

    public IslandSurvivalCraft() {
        worldInfoManager = new WorldInfoManager();
    }
    
    @Override
    public void onEnable() {
        pluginManager = getServer().getPluginManager();
        pluginManager.registerEvents(worldInfoManager, this);
        commandProcessor = new CommandProcessor(this);
        super.onEnable();
    }

    @Override
    public void onDisable() {
        HandlerList.unregisterAll(worldInfoManager);
        commandProcessor = null;
        super.onDisable();
    }

    @Override
    public ChunkGenerator getDefaultWorldGenerator(String worldName, String id) {
        GeneratorModes gID = null;
        try {
            gID = GeneratorModes.valueOf(id);
        } catch (NullPointerException | IllegalArgumentException e) {
            gID = GeneratorModes.UNIQUE;
        }
        return worldInfoManager.getChunkGenerator(getServer().getWorld(worldName), gID);
    }

    /**
     * @return the world information manager.
     */
    public WorldInfoManager getWorldInfoManager() {
        return worldInfoManager;
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        return commandProcessor.onCommand(sender, command, label, args);
    }
}
