package ca.recrown.islandsurvivalcraft.interaction.commands.runnables;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.concurrent.CompletableFuture;

import org.bukkit.Bukkit;
import org.bukkit.Color;
import org.bukkit.Location;
import org.bukkit.Particle;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;
import ca.recrown.islandsurvivalcraft.utilities.datatypes.Point2;
import ca.recrown.islandsurvivalcraft.utilities.pathfinding.DepthFirstSearch;
import ca.recrown.islandsurvivalcraft.world.IslandWorldMap;
import ca.recrown.islandsurvivalcraft.world.WorldInfo;

public class HighlightIslandCommand implements CommandRunnable {
    private final Particle.DustOptions OPTIONS = new Particle.DustOptions(Color.RED, 1f);
    private final int PARTICLE_AMOUNT = 3;
    private final double OFFSET_X = 0.0d, OFFSET_Y = 6.0d, OFFSET_Z = 0.0d;
    private final double EXTRA = 0d;
    HashSet<Player> playersHighlighting = new HashSet<>();
    LinkedList<PlayerIslandHighlighter> waitingList = new LinkedList<>();

    @Override
    public String getDescription() {
        return "Displays particle effects at edge of islands.";
    }

    @Override
    public boolean invoke(CommandSender sender, String[] args) {
        if (!(sender instanceof Player)) {
            sender.sendMessage("This command can only be run as a player!");
            return false;
        }
        Player player = (Player) sender;
        if (args[0].isEmpty()) {
            return false;
        }
        if (args[0].toLowerCase().equals("start")) {
            if (playersHighlighting.add(player)) {
                waitingList.add(new PlayerIslandHighlighter(player));
                sender.sendMessage("You are now highlighting islands.");
            } else {
                sender.sendMessage("You are already highlighting islands.");
            }
        } else if (args[0].toLowerCase().equals("stop")) {
            if (playersHighlighting.remove(player)) {
                sender.sendMessage("You are no longer highlighting islands.");
            } else {
                sender.sendMessage("You weren't highlighting islands.");
            }
        } else {
            return false;
        }
        return true;
    }

    @Override
    public void initialize(IslandSurvivalCraft islandsurvivalcraft) {        
        Bukkit.getScheduler().scheduleSyncRepeatingTask(islandsurvivalcraft, new Runnable() {
            @Override
            public void run() {
                PlayerIslandHighlighter highlighter = waitingList.poll();
                if (highlighter == null) return;
                Player player = highlighter.player;
                if (player == null || !player.isOnline() || !playersHighlighting.contains(player)) {
                    playersHighlighting.remove(player);
                    return;
                }
                World world = player.getWorld();
                WorldInfo worldInfo = islandsurvivalcraft.getWorldInfoManager().retrieve(world);
                Location playerLocation = player.getLocation();
                int playerX = playerLocation.getBlockX(), playerY = playerLocation.getBlockY(), playerZ = playerLocation.getBlockZ();
                IslandWorldMap map = worldInfo.getIslandMap();
                
                //TODO SPAWN HIGHLIGHTING PARTICLES.
                waitingList.add(highlighter);
            }
        }, 0, 8);

    }

    private void spawnParticle(World world, double x, double y, double z) {
        world.spawnParticle(Particle.REDSTONE, x, y, z, PARTICLE_AMOUNT, OFFSET_X, OFFSET_Y, OFFSET_Z, EXTRA, OPTIONS, true);
    }

    @Override
    public String getDetailedDescription() {
        return "A helpful debugging command to analyze what the plugin considers an island by displaying particle effects around the player that is a part of the island. The <start | stop> argument is required to dicate whether or not to start the highlighting or stop it.";
    }
    
    private class PlayerIslandHighlighter {
        private final int MAX_SEARCH = 0;
        private final Player player;
        private DepthFirstSearch dfs;

        public PlayerIslandHighlighter(Player player) {
            this.player = player;
        }

        public CompletableFuture<ArrayList<Point2>> update(Point2 playerLocation, IslandWorldMap islandWorldMap, IslandSurvivalCraft isc) {
            CompletableFuture<ArrayList<Point2>> completableFuture = new CompletableFuture<>();
            Bukkit.getScheduler().runTaskAsynchronously(isc, () -> {
                ArrayList<Point2> border = new ArrayList<>(64);
                while (dfs == null || dfs.step(islandWorldMap.islandValidator, (p) -> false)) {
                    dfs = new DepthFirstSearch(playerLocation, null, (p1, p2) -> {
                        double diff = islandWorldMap.getWorldValue(p1.x, p1.y) - islandWorldMap.getWorldValue(p2.x, p2.y);
                        if (diff == 0) {
                            return 0;
                        } else if (diff > 0) {
                            return 1;
                        } else {
                            return -1;
                        }
                    }, MAX_SEARCH);
                    Point2 currentPoint = dfs.getCurrentNode();
                    if (currentPoint != null && islandWorldMap.isEdgeOfIsland(currentPoint.x, currentPoint.y)) {
                        border.add(currentPoint);
                    }
                }
                completableFuture.complete(border);
            });

            return completableFuture;
        }
    }
}